@@ -8,10 +8,12 @@ Implementation for of the configuration object.
 """
 
 import os,time,socket,sys
-from data import *
-import base_classes
-import themes
-from error import log_scapy
+
+from scapy import VERSION
+from scapy.data import *
+from scapy import base_classes
+from scapy import themes
+from scapy.error import log_scapy
 
 ############
 ## Config ##
@@ -23,11 +25,10 @@ class ConfClass(object):
     def __repr__(self):
         return str(self)
     def __str__(self):
-        s=""
+        s = ""
         keys = self.__class__.__dict__.copy()
         keys.update(self.__dict__)
-        keys = keys.keys()
-        keys.sort()
+        keys = sorted(keys)
         for i in keys:
             if i[0] != "_":
                 r = repr(getattr(self, i))
@@ -130,13 +131,15 @@ class Num2Layer:
                 dir = "<->"
             else:
                 dir = " ->"
-            lst.append((num,"%#6x %s %-20s (%s)" % (num,dir,layer.__name__,layer.name)))
+            lst.append((num,"%#6x %s %-20s (%s)" % (num, dir, layer.__name__,
+                                                    layer._name)))
         for layer,num in self.layer2num.iteritems():
             if num not in self.num2layer or self.num2layer[num] != layer:
-                lst.append((num,"%#6x <-  %-20s (%s)" % (num,layer.__name__,layer.name)))
+                lst.append((num,"%#6x <-  %-20s (%s)" % (num, layer.__name__,
+                                                         layer._name)))
         lst.sort()
         return "\n".join(y for x,y in lst)
-            
+
 
 class LayersList(list):
     def __repr__(self):
@@ -278,7 +281,7 @@ def _prompt_changer(attr,val):
     prompt = conf.prompt
     try:
         ct = val
-        if isinstance(ct, AnsiColorTheme) and ct.prompt(""):
+        if isinstance(ct, themes.AnsiColorTheme) and ct.prompt(""):
             ## ^A and ^B delimit invisible caracters for readline to count right.
             ## And we need ct.prompt() to do change something or else ^A and ^B will be
             ## displayed
@@ -316,8 +319,9 @@ resolve   : holds list of fields for which resolution should be done
 noenum    : holds list of enum fields for which conversion to string should NOT be done
 AS_resolver: choose the AS resolver class to use
 extensions_paths: path or list of paths where extensions are to be looked for
+contribs: a dict which can be used by contrib layers to store local configuration
 """
-    version = "2.3.2"
+    version = VERSION
     session = ""
     interactive = False
     interactive_shell = ""
@@ -343,7 +347,10 @@ extensions_paths: path or list of paths where extensions are to be looked for
     L3socket = None
     L2socket = None
     L2listen = None
-    histfile = os.path.join(os.path.expanduser("~"), ".scapy_history")
+    min_pkt_size = 60
+    histfile = os.getenv('SCAPY_HISTFILE',
+                         os.path.join(os.path.expanduser("~"),
+                                      ".scapy_history"))
     padding = 1
     except_filter = ""
     debug_match = 0
@@ -358,8 +365,9 @@ extensions_paths: path or list of paths where extensions are to be looked for
     resolve = Resolve()
     noenum = Resolve()
     emph = Emphasize()
-    use_pcap = False
-    use_dnet = False
+    use_pcap = os.getenv("SCAPY_USE_PCAPDNET", "").lower().startswith("y")
+    use_dnet = os.getenv("SCAPY_USE_PCAPDNET", "").lower().startswith("y")
+    use_winpcapy = False
     ipv6_enabled = socket.has_ipv6
     ethertypes = ETHER_TYPES
     protocols = IP_PROTOS
@@ -371,11 +379,15 @@ extensions_paths: path or list of paths where extensions are to be looked for
     stats_dot11_protocols = []
     temp_files = []
     netcache = NetCache()
-    load_layers = ["l2", "inet", "dhcp", "dns", "dot11", "gprs", "hsrp", "inet6", "ir", "isakmp", "l2tp",
-                   "mgcp", "mobileip", "netbios", "netflow", "ntp", "ppp", "radius", "rip", "rtp",
-                   "sebek", "skinny", "smb", "snmp", "tftp", "x509", "bluetooth", "dhcp6", "llmnr", "sctp", "vrrp",
-                   "ipsec", "loop" ]
-    
+    geoip_city = '/usr/share/GeoIP/GeoLiteCity.dat'
+    load_layers = ["l2", "inet", "dhcp", "dns", "dot11", "gprs", "tls",
+                   "hsrp", "inet6", "ir", "isakmp", "l2tp", "mgcp",
+                   "mobileip", "netbios", "netflow", "ntp", "ppp",
+                   "radius", "rip", "rtp", "skinny", "smb", "snmp",
+                   "tftp", "x509", "bluetooth", "dhcp6", "llmnr",
+                   "sctp", "vrrp", "ipsec", "lltd", "vxlan", "loop"]
+    contribs = dict()
+
 
 if not Conf.ipv6_enabled:
     log_scapy.warning("IPv6 support disabled in Python. Cannot load scapy IPv6 layers.")
