@@ -3,12 +3,21 @@
 ## Copyright (C) Philippe Biondi <phil@secdev.org>
 ## This program is published under a GPLv2 license
 
-import re,struct
+"""
+Wireless LAN according to IEEE 802.11.
+"""
 
+import re,struct
+from zlib import crc32
+
+from scapy.config import conf
+from scapy.data import *
 from scapy.packet import *
 from scapy.fields import *
+from scapy.ansmachine import *
 from scapy.plist import PacketList
 from scapy.layers.l2 import *
+from scapy.layers.inet import IP, TCP
 
 
 try:
@@ -121,6 +130,15 @@ class RadioTap(Packet):
                                                      'b24','b25','b26','b27','b28','b29','b30','Ext']),
                     StrLenField('notdecoded', "", length_from= lambda pkt:pkt.len-8) ]
 
+class PPI(Packet):
+    name = "Per-Packet Information header (partial)"
+    fields_desc = [ ByteField("version", 0),
+                    ByteField("flags", 0),
+                    FieldLenField("len", None, fmt="<H", length_of="fields", adjust=lambda pkt,x:x+8),
+                    LEIntField("dlt", 0),
+                    StrLenField("notdecoded", "", length_from = lambda pkt:pkt.len-8)
+                    ]
+
 
 
 class Dot11SCField(LEShortField):
@@ -154,7 +172,7 @@ class Dot11(Packet):
     def mysummary(self):
         return self.sprintf("802.11 %Dot11.type% %Dot11.subtype% %Dot11.addr2% > %Dot11.addr1%")
     def guess_payload_class(self, payload):
-        if self.type == 0x02 and (self.subtype >= 0x08 and self.subtype <=0xF and self.subtype != 0xD):
+        if self.type == 0x02 and (0x08 <= self.subtype <= 0xF and self.subtype != 0xD):
             return Dot11QoS
 	elif self.FCfield & 0x40:
             return Dot11WEP
@@ -338,6 +356,7 @@ class Dot11WEP(Packet):
 
 bind_layers( PrismHeader,   Dot11,         )
 bind_layers( RadioTap,      Dot11,         )
+bind_layers( PPI,           Dot11,         dlt=105)
 bind_layers( Dot11,         LLC,           type=2)
 bind_layers( Dot11QoS,      LLC,           )
 bind_layers( Dot11,         Dot11AssoReq,    subtype=0, type=0)
@@ -367,6 +386,7 @@ conf.l2types.register_num2layer(801, Dot11)
 conf.l2types.register(119, PrismHeader)
 conf.l2types.register_num2layer(802, PrismHeader)
 conf.l2types.register(127, RadioTap)
+conf.l2types.register(0xc0, PPI)
 conf.l2types.register_num2layer(803, RadioTap)
 
 
@@ -426,11 +446,11 @@ iwconfig wlan0 mode managed
         p /= self.replace
         q.ID += 1
         q.getlayer(TCP).flags="RA"
-        q.getlayer(TCP).seq+=len(replace)
+        q.getlayer(TCP).seq+=len(self.replace)
         return [p,q]
     
     def print_reply(self):
-        print p.sprintf("Sent %IP.src%:%IP.sport% > %IP.dst%:%TCP.dport%")
+        print self.sprintf("Sent %IP.src%:%IP.sport% > %IP.dst%:%TCP.dport%")
 
     def send_reply(self, reply):
         sendp(reply, iface=self.ifto, **self.optsend)
