@@ -3,7 +3,12 @@
 ## Copyright (C) Philippe Biondi <phil@secdev.org>
 ## This program is published under a GPLv2 license
 
+"""
+ISAKMP (Internet Security Association and Key Management Protocol).
+"""
+
 import struct
+from scapy.config import conf
 from scapy.packet import *
 from scapy.fields import *
 from scapy.ansmachine import *
@@ -161,7 +166,7 @@ class ISAKMP_class(Packet):
     def guess_payload_class(self, payload):
         np = self.next_payload
         if np == 0:
-            return Raw
+            return conf.raw_layer
         elif np < len(ISAKMP_payload_type):
             pt = ISAKMP_payload_type[np]
             return globals().get("ISAKMP_payload_%s" % pt, ISAKMP_payload)
@@ -184,7 +189,7 @@ class ISAKMP(ISAKMP_class): # rfc2408
 
     def guess_payload_class(self, payload):
         if self.flags & 1:
-            return Raw
+            return conf.raw_layer
         return ISAKMP_class.guess_payload_class(self, payload)
 
     def answers(self, other):
@@ -242,7 +247,7 @@ class ISAKMP_payload_Proposal(ISAKMP_class):
         FieldLenField("SPIsize",None,"SPI","B"),
         ByteField("trans_nb",None),
         StrLenField("SPI","",length_from=lambda x:x.SPIsize),
-        PacketLenField("trans",Raw(),ISAKMP_payload_Transform,length_from=lambda x:x.length-8),
+        PacketLenField("trans",conf.raw_layer(),ISAKMP_payload_Transform,length_from=lambda x:x.length-8),
         ]
 
 
@@ -275,7 +280,7 @@ class ISAKMP_payload_SA(ISAKMP_class):
         FieldLenField("length",None,"prop","H", adjust=lambda pkt,x:x+12),
         IntEnumField("DOI",1,{1:"IPSEC"}),
         IntEnumField("situation",1,{1:"identity"}),
-        PacketLenField("prop",Raw(),ISAKMP_payload_Proposal,length_from=lambda x:x.length-12),
+        PacketLenField("prop",conf.raw_layer(),ISAKMP_payload_Proposal,length_from=lambda x:x.length-12),
         ]
 
 class ISAKMP_payload_Nonce(ISAKMP_class):
@@ -327,14 +332,13 @@ class ISAKMP_payload_Hash(ISAKMP_class):
 
 
 ISAKMP_payload_type_overload = {}
-for i in range(len(ISAKMP_payload_type)):
-    name = "ISAKMP_payload_%s" % ISAKMP_payload_type[i]
+for i, payloadname in enumerate(ISAKMP_payload_type):
+    name = "ISAKMP_payload_%s" % payloadname
     if name in globals():
-        ISAKMP_payload_type_overload[globals()[name]] = {"next_payload":i}
+        ISAKMP_payload_type_overload[globals()[name]] = {"next_payload": i}
 
-del(i)
-del(name)
-ISAKMP_class.overload_fields = ISAKMP_payload_type_overload.copy()
+del i, payloadname, name
+ISAKMP_class._overload_fields = ISAKMP_payload_type_overload.copy()
 
 
 bind_layers( UDP,           ISAKMP,        dport=500, sport=500)
